#ifndef __HAPLOTYPE_H__
#define __HAPLOTYPE_H__

#include <map>

#include "shaunlib.h"
#include "window.h"

class CParentalPhase
{
public:
//    double prob;
    CParentalPhase();
    CPhase father;
    CPhase mother;
    vector<CPhase> P;
    int mult;    // number of times this configuration appears 
                 // (used for redundancy in EveryTrio)
};


class CBox 
{
 public:

    // default constructor
    CBox(CIndividual *p);

    // Pointer to owner
    CIndividual *person;

    // Are parents genotyped?
    bool parents;

    // Is this individual to be analysed, or a parent?
    bool proband;

    // Phase probabilities
    
    // Not conditional on parents
    vector<CPhase> P;  

    // Conditional on parents
    vector<CParentalPhase> PP;

    // Between and Within components
    // Phase x Haplotype definition matrix (reconstruct from contained in K, P now)
    // Expand for all possible phases (conditional on parents or not)
    CMatrix B;
    CMatrix W;
    
    CColVector K;

    // Note -> if not conditioning on parents, and singletons, then
    // K is P()->prob, and B()() is calculated from P()->phase

    // If conditioning on parents, then B and W are extracted from PP()
       
    // Permutation code
    int pcode;

    // CHECK: probably don't now need to explicitly keep track of this...
    int nphase;


    // Functions (repeats functions from CIndividual..)
    void enumeratePhase(Window &);
    int  nPhase() { return nphase; }
    void addPhase(CHaplotype pat, CHaplotype mat, double prob = 0);
    void removePhase(int);
    void clearPhase();
    CPhase & phase(const int n)
	{ return P[n-1]; }
    vector<CPhase> & phase()
	{ return P; }

};


class ind
{
 public:
    // Sample haplotype names / frequencies
    
    static vector<CHaplotype> hap;       // zero-based
    static CVector hapfreq;              // zero-based
    static CVector hapfreq1;             // sum to 1 
    static map<CHaplotype, int> hcode;   // reference for B and W positions

    static BoolArray alt_estimate;       // loci to estimate under the alternate
    static BoolArray null_estimate;      // loci to estimate under the null    
    
    static int nloci;                    // number of loci
    static int nsnps;                    // number of SNPs (i.e. if downcoded usats)
    static IntArray inc_loci;            // list of loci in --alt

};

#endif 
