#ifndef __PARAMETERS_H__
#define __PARAMETERS_H__

class par {
    
 public:

    static double threshold;       // parameter to pass to SNPHAP
    static bool binary;
    static bool debug;             // turn on DEBUG mode
    static bool weight;            // perform weighted regression analysis
    static double abs_threshold;   // sample freq threshold for haplotype 
    static bool readhaps;          // if true, haplotypes already calculated
    static bool calcfullhaps;      // if true, send all markers to SNPHAP, 
                                   // otherwise just send --alt markers
    static double eps;             // small value for calculation of Hessian
    static int perm;               // number of permutations in permutation test 
    static bool gperm;             // permute within-genotype (cf. permute trait (TDT))
    static bool tperm;             // permute trait (can combine with --gperm)
    static int sec_perm;           // number of permutations in secondary p test

    static bool hap_specific;      // instead of omnibus test, get the largest single hap result
    static bool hs_min_output;     // minimalist HS output

    static bool xchr;              // data are from the X chromosome

    static double tol;             // whap optimisation values -- tolerance
    static int mxiter;             // and maximum number of iterations

    static int sa_temp;            // simulated annealing parameters (default =10)
    static int sa_cycle;           // default = 10
    static double sa_decline;      // default = 0.8
    static int sa_mxiter;          // default = 25
    static int sa_repeat;          // # of SA/Simplex cycles: default =1

    static bool verbose;           // convergence information
    static bool dump;              // dump phase information
    static bool nice_dump;         // dump phase information, human readable
    static bool dump_scores;       // dump B and W scoring

    static int covn;               // number of covariates (under --alt and --null)
    static bool alt_cov;           // covariates under the alternate
    static bool null_cov;          // covariates under the null
    static bool alt_gxe;           // GxE under alt
    static bool null_gxe;          // GxE under null

    static bool wald;              // perform Wald tests -- disregarded
    static bool ci;                // get C.I.s on parameter estimates
    static double civ;             // percent for CI default = 0.95
    
    static bool secondary;         // perform secondary beta analysis
    static bool secondary_verbose; // give more output regarding secondary analysis

    static bool window;            // perform sliding window analysis
    static int winstep;            // number of markers to shift each window
    static bool win_all_results;   // display full results even when in --window
    static bool prepermute;        // keep phenotype constant for whole sliding win

    static bool joint;             // Joint estimation of haplotype frequencies L(X,H) 
    static bool cond;              // Conditional estimation L(H|X)

    static int recomb;             // Max number of recombinations to allow in a hap 

    static bool alt_po;            // allow parent-origin effect under alternate
    static bool null_po;           // allow under null as well

    static bool alt_between;       // Model specifications
    static bool alt_within;
    static bool alt_total;
    static bool null_between;
    static bool null_within;
    static bool null_total;

    static bool usr_constraint;   // User-specified haplotype constraints

    static bool fix_mean;         // If fixing mean/prevalence, variance
    static bool fix_var;
    static double fixed_mean;
    static double fixed_var;

    static bool no_between_trios; 
    static bool no_unrel;          // Set to true if no unrels in sample, for speed-up
    static bool no_trios;          // Set to true if no trios in sample, for speed-up
    static bool qual_cond_speedup; // Only calculate denominator twice each function evaluation (0/1)
    static bool collapse_phase_speedup; // Collapse K, B and W for equal rows of K
    static int uninform_threshold;      // if a proband has this many, or more phases, remove
    static long int bp_gap;             // definition of a gap in base-pairs, to be jumped by --window 
    static bool usat;                   // look out for downcoded usats, deal w/ appropriately...
    static bool sec_global;             // use global/LL in window mode (secondary test)
    static bool model_parental_phenos;  // if non-missing, treat as dependend variables  parental QT phenos
    static bool keep_snphap_files;      // copy to files the input and output SNPHAP files

    static bool one_sided_pos;          // perform a one-sided test for 1 df LRTs: effect is +ve
    static bool one_sided_neg;          // perform a one-sided test for 1 df LRTs: effect is -ve
    static bool model_dom;              // dominant model (hap-specific only)
    static bool model_rec;              // recessive model (hap-specific only)
    
};


#endif
