#include <iostream> 
#include "whap.h"
#include "function.h"
#include "model.h"
#include "results.h"
#include "parameters.h"
#include "helper.h"
#include "secondary.h"

using namespace std;

WindowResults whap(Driver & drive, int nhap)
{

    Model alt, null;
    Model perm;      
    WindowResults w;
    

    ////////////////////////////////////////////////////////////////////////
    // Calculate number of unique haplotypes (in the potentially reduced set)

    // alt_betacode and alt_nsubhap will be changed
    if (!par::usr_constraint )
	helper::calcUniqueReducedHaplotypeNum(ind::alt_estimate, nhap, 
					      alt.betacode, alt.nsubhap,drive);
    else
	helper::calcUserReducedHaplotypeNum(drive.usr_alt, nhap, 
					    alt.betacode, alt.nsubhap);

    
    /////////////////////////////////////////
    // Count number of parameters in model
    // (Mean) (Var) (Cov betas) (Bet Betas) (With Betas) (Freqs)
    alt.total = par::alt_total;
    alt.between = par::alt_between;
    alt.within = par::alt_within;
    alt.cov = par::alt_cov;
    alt.gxe = par::alt_gxe;

    alt.nparam = alt.nsubhap - 1;
    if (alt.between && alt.within && !alt.total) alt.nparam *= 2;  // haplotype effects
    if (par::binary && !par::fix_mean) alt.nparam++;               // prevalence 
    if (!par::binary && !par::fix_mean) alt.nparam++;              // mean
    if (!par::binary && !par::fix_var) alt.nparam++;               // variance
    if (alt.cov) alt.nparam += par::covn;                          // covariates
    if (alt.gxe) alt.nparam += par::covn*(alt.nsubhap-1);          // matrix of moderator terms
    if (par::joint) alt.nparam += nhap;                            // freqs for joint estimation


    ////////////////////////////////////////////////////////////////////////
    // Calculate number of unique haplotypes (in the potentially reduced set)
    // (Repeat for null) 

    // null_betacode and null_nsubhap will be changed
    if (!par::usr_constraint)
	helper::calcUniqueReducedHaplotypeNum(ind::null_estimate, nhap, 
					      null.betacode, null.nsubhap,drive);
    else
	helper::calcUserReducedHaplotypeNum(drive.usr_null, nhap, 
					    null.betacode, null.nsubhap);
    

    ///////////////////////////
    // Count parameters
    null.total = par::null_total;
    null.between = par::null_between;
    null.within = par::null_within;
    null.cov = par::null_cov;
    null.gxe = par::null_gxe;

    null.nparam = null.nsubhap - 1;
    if (null.between && null.within && !null.total) null.nparam *= 2;  // haplotype effects
    if (par::binary && !par::fix_mean) null.nparam++;                  // prevalence 
    if (!par::binary && !par::fix_mean) null.nparam++;                 // mean
    if (!par::binary && !par::fix_var) null.nparam++;                  // variance
    if (null.cov) null.nparam += par::covn;                            // covariates
    if (null.gxe) null.nparam += par::covn*(null.nsubhap-1);           // interaction terms
    if (par::joint) null.nparam += nhap;                               // freqs for joint estimation

    //////////////////////////////
    // Create model for alternate
    
    alt.estimate().resize(ind::nloci);
    for (int i=1; i<=ind::nloci; i++)
	alt.estimate(i) = ind::alt_estimate(i);
    alt.nhap = nhap;


    //////////////////////////////
    // Create model for null
    
    null.estimate().resize(ind::nloci);
    for (int i=1; i<=ind::nloci; i++)
	null.estimate(i) = ind::null_estimate(i);
    null.nhap = nhap;


    ////////////////////////////////////////////
    // Create K, B and W for unconditional analysis

    // NB. it does not matter whether Model::alt or Model::null is specified
    for (int f=1; f<=drive.sample.countFamily(); f++)
    {

	if (par::debug) cout << "\n ++ About the score " 
			     << drive.sample.fam(f)->proband->family->getID() 
			     << " ++ \n";

	if (!drive.sample.fam(f)->proband->box->parents)
	    drive.scoreWithoutParents(drive.sample.fam(f)->proband->box->P,
				      drive.sample.fam(f)->proband->box->K,
				      drive.sample.fam(f)->proband->box->B,
				      drive.sample.fam(f)->proband->box->W,
				      drive.sample.fam(f)->proband->box->nphase,
				      nhap,
				      drive.sample.fam(f)->proband,
				      par::uninform_threshold);

	else
	    drive.scoreWithParents(drive.sample.fam(f)->proband->box->PP,
				   drive.sample.fam(f)->proband->box->K,
				   drive.sample.fam(f)->proband->box->B,
				   drive.sample.fam(f)->proband->box->W,
				   drive.sample.fam(f)->proband->box->nphase,
				   nhap,
				   drive.sample.fam(f)->proband,
				   par::uninform_threshold);
	
    }
    
    //////////////////////////////////////////////////////////
    // Create everyman and everytrio for conditional analysis
    
    if (par::cond)
    {	
	drive.everyman = new CIndividual(&drive.sample);
	drive.everytrio = new CIndividual(&drive.sample);
	drive.everyman->box.set(new CBox(drive.everyman));
	drive.everytrio->box.set(new CBox(drive.everytrio));
	drive.everyman->box->parents = false;
	drive.everytrio->box->parents = true;
	drive.everyman->setID("EveryMan");
	drive.everytrio->setID("EveryTrio");
	
	////////////////////////////////////////////
	// Create K, B and W for conditional analysis
	
	if (!par::no_unrel)
	    drive.poss = drive.allPossibleWithoutParents(ind::nloci,nhap);
	if (!par::no_trios)
	    drive.pposs = drive.allPossibleWithParents(ind::nloci,nhap);
	
	if (par::debug) cout << "\n ++ About the score denominator ++ \n";
	
	if (!par::no_unrel)
	    drive.scoreWithoutParents(drive.poss,
				      drive.everyman->box->K,
				      drive.everyman->box->B,
				      drive.everyman->box->W,
				      drive.everyman->box->nphase,
				      nhap,
				      drive.everyman,
				      0);
	if (!par::no_trios)
	    drive.scoreWithParents(drive.pposs,
				   drive.everytrio->box->K,
				   drive.everytrio->box->B,
				   drive.everytrio->box->W,
				   drive.everytrio->box->nphase,
				   nhap,
				   drive.everytrio,
				   0);
	
    }
    



    ///////////////////////////////////////////////
    // If in nice_dump mode, just print out the 
    // phases for each individual, human readable, 
    // then exit

    if (par::nice_dump)
    {
	helper::niceDumper(drive, drive.sample);
	exit(0);
    }

    if (par::debug)
	cerr << "about to loop through permutations...\n";
    
    ///////////////////////////////////////////////
    // Loop through iterations 'p' of permutation test
    //
    // when p==0, do not permute the data (i.e. actual analysis)
    
    // restore original dataset for this window
    int n= drive.sample.countFamily();
    if (par::binary)
	for (int i=1; i <= n ; i++)
	    drive.sample.fam(i)->proband->bt(1) = drive.original_bt[i-1];
    else
	for (int i=1; i <= n ; i++)
	    drive.sample.fam(i)->proband->qt(1) = drive.original_qt[i-1];
    
    
    for (int p=0; p <= par::perm ; p++)
    {
	
	///////////////////////////////////
	// Permute the trait if p > 0
	if (p > 0)
	{
	    // Permute on the fly, if not already done
	    if (!par::prepermute && par::tperm)
	    {
		// Create permutation list
		// ASSUMPTION: TRIOS or UNRELATEDS ONLY 
		int n= drive.sample.countFamily();
		vector<int> pcode(n);
		mymath::permute(pcode);
		
		vector<bool> disease;
		vector<double> trait;

		// Get copy of traits / diseases
		if (par::binary) {
		    disease.resize(n);
		    for (int i=1; i <= n; i++)
			disease[i-1] = drive.original_bt[i-1];
		} else {
		    trait.resize(n);
		    for (int i=1; i <= n ; i++)
			trait[i-1] = drive.original_qt[i-1];
		}

		if (par::binary)
		    for (int i=1; i <= n ; i++)
			drive.sample.fam(i)->proband->bt(1) = disease[pcode[i-1]-1];
		else
		    for (int i=1; i <= n ; i++)
			drive.sample.fam(i)->proband->qt(1) = trait[pcode[i-1]-1];
	    }
	    else if (par::tperm)
	    {
		// Trait permutation : pre-permutation
		// Copy pre-permuted version into sample[]
		int n= drive.sample.countFamily();
		if (par::binary)
		    for (int i=1; i <= n ; i++)
			drive.sample.fam(i)->proband->bt(1) = (short)drive.preperm[p][i];
		else
		    for (int i=1; i <= n ; i++)
			drive.sample.fam(i)->proband->qt(1) = drive.preperm[p][i];
		
	    }
	    
	    if (par::gperm)
	    {
		// Genotype permutation : permute W matrix

		// W will be regenerated for next window position, so
		// no need to save.
		
		for (int i=1; i<=drive.sample.countFamily();i++)
		    drive.sample.fam(i)->proband->box->W.multiply(drive.wpreperm[p][i]);
		
	    }
	    
	}
	
	
       	
	///////////////////////////////////
	// Evaluate model for the Alternate
	
	CFunction alt_func(drive.sample,alt,drive);
	CSASimplex alt_opt(&alt_func);
	CVector start(alt.nparam);
	alt_opt.setParam(alt.nparam);
	start.clear();
	
	// starting value for variance needed?
	if (!par::binary && !par::fix_var)
	{
	    // is the mean fixed?
	    if (par::fix_mean) start[1] = drive.sample.qt(1)->var;
	    else start[2] = drive.sample.qt(1)->var;
	}

	alt_opt.setStart(start);
	alt_opt.verbose = par::verbose;
	alt_opt.semiverbose = false;
	alt_opt.setSimplexTol(par::tol);
	alt_opt.setSimplexMaxIter(par::mxiter);
	alt_opt.setSATemp(par::sa_temp);
	alt_opt.setSACycle(par::sa_cycle);
	alt_opt.setSADecline(par::sa_decline);
	alt_opt.setSAMaxIter(par::sa_mxiter);
	alt_opt.setTryHard(par::sa_repeat);
	
	alt_opt.minimise();
	
	
	///////////////////////////////////
	// Evaluate model for the Null
	
	CFunction null_func(drive.sample,null,drive);
	CSASimplex null_opt(&null_func);
	
	null_opt.setParam(null.nparam);
	start.setCol(null.nparam);
	start.clear();
	
	// starting value for variance needed?
	if (!par::binary && !par::fix_var)
	{
	    // is the mean fixed?
	    if (par::fix_mean) start[1] = drive.sample.qt(1)->var;
	    else start[2] = drive.sample.qt(1)->var;
	}
	
	null_opt.setStart(start);
	null_opt.verbose = par::verbose;
	null_opt.semiverbose = false;
	null_opt.setSimplexTol(par::tol);
	null_opt.setSimplexMaxIter(par::mxiter);
	null_opt.setSATemp(par::sa_temp);
	null_opt.setSACycle(par::sa_cycle);
	null_opt.setSADecline(par::sa_decline);
	null_opt.setSAMaxIter(par::sa_mxiter);
	null_opt.setTryHard(par::sa_repeat);
	null_opt.minimise();
	
	
	///////////////////////////////////////////
	// Store parameter values in Model for 
	// original non-permuted data (p==0)
	if (p==0) 
	{
	    alt.estimates = alt_opt.getEstimates();
	    null.estimates = null_opt.getEstimates();
	    alt.likelihood = alt_opt.getLikelihood();
	    null.likelihood = null_opt.getLikelihood();
	}
	

	/////////////////////////////////
	// Store -2LL from this permutation
	if (nhap > 1) 
	{
	    w.primary.push_back(null_opt.getLikelihood() 
				- alt_opt.getLikelihood());
	}
	else
	{
	    w.primary.push_back(-1);
	}
	
	if (par::win_all_results && !par::hs_min_output) 
	    cerr << "\n  --> permutation " << p << " -2LL = " << 
		null_opt.getLikelihood() 
		- alt_opt.getLikelihood() << "\t";	
	
	

	///////////////////////////////////////////////////////
	// Get C.I.s on parameter estimates, for original only
	
	if (par::ci && p==0)
	{
	    
	    alt.covbeta = alt_func.hessian(alt.estimates,par::eps);
	    
	    if (alt.covbeta.singular() || abs(alt.covbeta.det()) < 1 ) 
		par::ci = false;

	    
	    if (par::ci)
	    {	    
		alt.covbeta.negate();
		alt.covbeta = alt.covbeta.inverse();
	    }
	}
	


	///////////////////////////////////////////
	// Perform secondary analysis on the betas
	
	// copy alt model to perm 
	perm = alt;
	perm.estimates = alt_opt.getEstimates();
	
	// get the covariance matrix of estimates, and check that they look 
	// stable. currently, this is is abs(det) > 1. there may be better ways 
	// of doing this / a better value to use.
	
	bool secondary_okay = true;
	if (par::secondary)
	{
	    perm.covbeta = alt_func.hessian(perm.estimates,par::eps);
	    
	    if (perm.covbeta.singular() || alt.nsubhap <= 2) 
	    {
		secondary_okay = false;
	    }
	    else
	    {
		if ( abs(perm.covbeta.det()) < 1 ) secondary_okay = false;
	    }
	    
	    if (secondary_okay)
	    {	    
		perm.covbeta.negate();
		perm.covbeta = perm.covbeta.inverse();
	    }
	}

	////////////////////////////////////////
	// Extract parameters for alt and null
	
	// this function populates the Model members
	// from the Model.estimates() parameter vector
	
	helper::extractParameters(alt);
	helper::extractParameters(null);
	helper::extractParameters(perm);
	
	////////////////////////////////////////////////////////
	// Make subhaplotype -> haplotype conversion table 
	IntArray sub2full;
	helper::sub2fullMapping(alt,sub2full);
	helper::sub2fullMapping(perm,sub2full);
	
	if (par::secondary  && secondary_okay)
	{
	    vector<string> haplotype_list;
	    vector<double> beta_list;
	    
	    // get lists of haplotypes & betas
	    for (int i = 1 ; i <= perm.nsubhap ; i++) 
	    {
		string subhapname;
		for (int j=1; j<= ind::nloci; j++)
		{
		    if (perm.estimate(j)) subhapname += ind::hap[sub2full(i)].pos(j);
		    else subhapname += "-";
		}
		
		// Add haplotype string
		haplotype_list.push_back(subhapname);
		
		// Add beta (from current estimate, choose W over B 
		if (par::alt_within || par::alt_total)
		    beta_list.push_back(perm.with_beta[sub2full(i)]);
		else
		    beta_list.push_back(perm.bet_beta[sub2full(i)]);
	    }

	    // extract elements from covbeta 
	    // [mean] [var] [cov] [gxe] [bet] [with] [freqs]
	    // send between and within betas separately
	    // insert row and column of 0 for haplotype 1
	    // rem: covbeta contains subhap estimates 
	    
	    int start = 1;
	    if (!par::fix_mean) start++;                                   // skip mean/prevalence
	    if (!par::binary && !par::fix_var) start++;                    // skip variance
	    if (alt.cov) start += par::covn;                               // skip covariates
	    if (alt.gxe) start += par::covn*(alt.nsubhap-1);               // skip moderators
	    if (par::alt_between && par::alt_within && !par::alt_total)    // skip B if W available
		start += nhap-1;
	    
	    CMatrix extract_covar_beta(nhap,nhap);
	    
	    // nb. s1-start+1+1  -- the extra +1 is because haplotype 1 
	    // doesn't feature in betbeta (fixed to 0) but still must 
	    // be included
	    
	    extract_covar_beta.clear();
	    for (int s1 = start ; s1 <= start+(nhap-1-1) ; s1++)
		for (int s2 = start ; s2 <= start+(nhap-1-1) ; s2++)
		    extract_covar_beta[s1-start+1+1][s2-start+1+1]
			= perm.covbeta[s1][s2];
	    
	    /////////////////////////////////////////////////
	    // Perform and store secondary analysis on betas
	    
	    vector<double> secondary_walds;
	    secondary_walds = secondary_analysis(haplotype_list,
						 beta_list,
						 extract_covar_beta);
	    
	    w.secondary.push_back(secondary_walds);
	    
	    if (par::win_all_results) 
	    {
		cerr << " --> Secondary (global) = " << 
		    secondary_walds[0] << "\n";
	    }
	}
	else if (par::secondary && !secondary_okay)
	{
	    // If not valid for secondary test, store -999s
	    vector<double> dummy(ind::nloci);
	    for (int f=0; f < dummy.size(); f++)
		dummy[f] = -999;
	    w.secondary.push_back(dummy); 
	}
	

	///////////////////////////////////////
	// Let the user know something is happening...
	
	cerr << "\r" << p;
	cerr.flush();

    }
    cerr << "\r";
    cerr.flush();
    if (!par::hs_min_output) cout << "          ";
    

    
    //////////////////////////////////////////////////
    // Collect results and call output function
    // if not in window mode
    if (!par::window || (par::window && par::win_all_results))
      {
	if (!par::hs_min_output)
	  disp_results(drive,alt, null, w.primary, w.secondary);
	else
	  disp_hs_results(drive,alt, null, w.primary, w.secondary);
      }

    ///////////////////////////////////////
    // Try to do some cleaning up
    
    if (par::cond) 
    {
	drive.everyman->box->P.clear();
	drive.everyman->box->PP.clear();
	delete drive.everyman;
	delete drive.everytrio;
    }
    
    return w;
}

